@@ -83,6 +83,10 @@ class Queue(OrchestratorHTTP):
                 "SpecificContent": specific_content,
             }
         }
+        if not references:
+            format_body_start["transactionData"]["SpecificContent"]["ReferenceID"] = ran_uuid
+            format_body_start["transactionData"]["SpecificContent"]["BatchID"] = batch_id
+            format_body_start["transactionData"]["Reference"] = batch_id
         if references:
             try:
                 ref = ""
@@ -101,7 +105,8 @@ class Queue(OrchestratorHTTP):
             format_body_start["transactionData"]["SpecificContent"].update(fields)
 
         url = f"{self.base_url}{endpoint}"
-        return self._post(url, body=format_body_start)
+        data = self._post(url, body=format_body_start)
+        return QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, self.session, data["Id"], data["SpecificContent"], data["Reference"], data["Status"], self.access_token)
 
     def get_processing_records(self, num_days=1, options=None):
         """
@@ -153,7 +158,7 @@ class Queue(OrchestratorHTTP):
         elif not options:
             odata_filter = {}
         else:
-            odata_filter = {"$filter": f"QueueDefinitionId eq {self.id} and Status in ('Failed', 'Retried', 'Successful')"}
+            odata_filter = {"$filter": f"QueueDefinitionId eq {self.id} and Status in ('New', 'Abandoned', 'Retried', 'Successful')"}
         # print(odata_filter)
         if options is not None:
             for k, v in options.items():
@@ -163,14 +168,36 @@ class Queue(OrchestratorHTTP):
         query_params = urlencode(odata_filter)
         url = f"{self.base_url}{endpoint}?{query_params}"
         data = self._get(url)
-        # pprint(odata_filter)
-        # pprint(data)
-        # pprint(f"ODATA COUNT - -> {data['@odata.count']}")
-        filt_data = data['value']
+        odata_count = data["@odata.count"]
+        # print(f"@odata.count is {odata_count}")
+        logging.debug(f"@odata.count is {odata_count}")
+        values = data["value"]
+        count = len(values)
+        skip = 0
+        # if odata_count is higher, paginate through the items
+        i = 1
+        while count < odata_count:
+            logging.info("OData count is higher than the supported maximum number of items. Starting pagination")
+            if options is not None:
+                for k, v in options.items():
+                    odata_filter.update({k: v})
+            # skip formula
+            # print("Skip "+str(skip))
+            skip += 1000
+            odata_filter.update({"$skip": skip})
+            query_params = urlencode(odata_filter)
+            url = f"{self.base_url}{endpoint}?{query_params}"
+            aux_data = self._get(url)["value"]
+            values += aux_data
+            count = len(values)
+            i += 1
+        logging.info(f"Length of the requested list of items is: {len(values)}")
+
+        # filt_data = data['value']
         if len(odata_filter) < 2:
-            return [QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, session=self.session, item_id=item["Id"], content=item["SpecificContent"], reference=item["Reference"], status=item["Status"], access_token=self.access_token) for item in filt_data]
+            return [QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, session=self.session, item_id=item["Id"], content=item["SpecificContent"], reference=item["Reference"], status=item["Status"], access_token=self.access_token) for item in values]
         else:
-            return filt_data
+            return values
 
     def filter_by_reference(self, reference, num_days=2):
         """
@@ -183,19 +210,25 @@ class Queue(OrchestratorHTTP):
         filt_options = {
             "$select": f"SpecificContent/{reference}, CreationTime, Status"
         }
+
         items = self.get_queue_items(options=filt_options)
         try:
             aux_list = [[i["SpecificContent"][reference], i["CreationTime"], i["Status"]] for i in items]
-            print(f"Length of aux_list: {len(aux_list)}")
+            logging.info(f"Length of aux_list: {len(aux_list)}")
+            INDEXES = []
             for i, item in enumerate(aux_list):
                 creation_date = item[1]
                 try:
                     diff = tod - datetime.timedelta(days=datetime.datetime.strptime(creation_date, '%Y-%m-%dT%H:%M:%S.%fZ').day)
                 except ValueError:
+                    # raise
                     diff = tod - datetime.timedelta(days=datetime.datetime.strptime(creation_date, '%Y-%m-%dT%H:%M:%SZ').day)
-
-                if diff.day > num_days:
-                    aux_list.pop(i)
+                if diff.day > num_days and diff.day != 31:
+                    INDEXES.append(i)
+            if INDEXES:
+                for index in sorted(INDEXES, reverse=True):
+                    del aux_list[index]
+            logging.info(f"There is a total of {len(aux_list)} items")
             return aux_list
         except KeyError as err:
             raise err
@@ -243,7 +276,7 @@ class Queue(OrchestratorHTTP):
             item whose reference matches the one indicated as an argument. Otherwise it returns
             False.
         """
-        filt_items = self.get_queue_items(options={"$filter": f"contains(Reference, '{reference}') and Status eq 'Successful'"})
+        filt_items = self.get_queue_items(options={"$filter": f"contains(Reference, '{reference}') and Status in ('Successful', 'Failed')"})
 
         if len(filt_items) > 0:
             return filt_items[0]
@@ -266,7 +299,7 @@ class Queue(OrchestratorHTTP):
             ids.append({item.id: item.queue_name})
         return ids
 
-    def add_queue_item(self, specific_content=None, priority="Low"):
+    def add_queue_item(self, specific_content=None, priority="Low", reference=None):
         """Creates a new Item
 
             @specific_content: dictionary of key value pairs (it does not
@@ -279,17 +312,26 @@ class Queue(OrchestratorHTTP):
         url = f"{self.base_url}{endpoint}{uipath_svc}"
         if not specific_content:
             raise OrchestratorMissingParam(value="specific_content", message="specific content cannot be null")
+        if reference:
+            try:
+                ref = specific_content[reference]
+                ref_uuid = f"{ref}#{str(uuid4())}"
+            except KeyError:
+                raise Exception(f"Key {reference} not found")
+        else:
+            ref_uuid = str(uuid4())
         format_body_queue = {
             "itemData": {
                 "Priority": priority,
                 "Name": self.name,
                 "SpecificContent": specific_content,
-                "Reference": self.generate_reference(),
+                "Reference": ref_uuid,
                 # "Progress": "In Progres"
             }
         }
         # pprint(format_body_queue)
-        return self._post(url, body=format_body_queue)
+        data = self._post(url, body=format_body_queue)
+        return QueueItem(self.client_id, self.refresh_token, self.tenant_name, self.folder_id, self.folder_name, self.name, self.id, self.session, data["Id"], data["SpecificContent"], data["Reference"], data["Status"], access_token=self.access_token)
 
     def _format_specific_content(self, sp_content, reference=None, priority="Low", progress="New", batch_id=None):
         ran_uuid = str(uuid4())
@@ -365,7 +407,6 @@ class Queue(OrchestratorHTTP):
         """
         endpoint = "/Queues"
         uipath_svc = "/UiPathODataSvc.BulkAddQueueItems"
-        print(type(specific_contents[0]))
         url = f"{self.base_url}{endpoint}{uipath_svc}"
         if not specific_contents:
             raise OrchestratorMissingParam(value="specific_contents", message="specific contents cannot be null")
@@ -379,7 +420,7 @@ class Queue(OrchestratorHTTP):
         # pprint(format_body_queue)
         return self._post(url, body=format_body_queue)
 
-    def edit_queue(self, name=None, description=None):
+    def edit_queue(self, name, description=None):
         """Edits the queue with a new name and a new
         descriptions
 
@@ -387,14 +428,15 @@ class Queue(OrchestratorHTTP):
         @description: the new description of the queue
 
         """
-        if not name or not description:
-            raise OrchestratorMissingParam(value="name/description", message="name and/or description cannot be null")
+        if not description:
+            raise OrchestratorMissingParam(value="description", message="description cannot be null")
         endpoint = f"/QueueDefinitions({self.id})"
         url = f"{self.base_url}{endpoint}"
         format_body_queue = {
             "Name": name,
             "Description": description
         }
+        pprint(format_body_queue)
         return self._put(url, body=format_body_queue)
 
     def delete_queue(self):
